/*
 * fd_objects.c
 *
 * This file is part of the Form Designer.
 *
 * It contains routines to keep track of all different object classes
 * and the types inside each class. To add a class of object to the
 * form designer, only this file has to be changed slightly:
 * 	- in init_classes() the class and types must be added.
 *	- in add_an_object() the way of adding it should be added.
 *
 * Written by Mark Overmars
 *
 * Version 2.1b
 * Date  May 27, 1992
 */

#include <string.h>
#include <stdio.h>
#include "forms.h"
#include "fd_main.h"
#include "fd_theforms.h"

/****
  THE DATA STRUCTURE
****/

typedef struct {
  int defined;
  char tname[40];
} TDEF;

typedef struct {
  int  cn;
  char cname[40];
  TDEF types[MAXTYPE];
  FL_OBJECT *defobj;
} CDEF;

static CDEF cl[MAXCLASS];
static int cnumb = 0;

typedef FL_OBJECT *(*FL_ADDPTR)(int, float, float, float, float, char []);

static void add_class_def(int numb, char name[], FL_ADDPTR createit)
/* Adds a class definition */
{
  int i;
  if (cnumb == MAXCLASS) return;
  cl[cnumb].cn = numb;
  strcpy(cl[cnumb].cname,name);
  cl[cnumb].defobj = (*createit)(1,0.0,0.0,1.0,1.0,"");
  for (i=0; i<MAXTYPE;i++) cl[cnumb].types[i].defined = FALSE;
  cnumb++;
}

static void add_type_def(int cn,int tn,char name[])
/* Adds a type definition to a class */
{
  int i;
  if (tn >= MAXTYPE) return;
  for (i=0; i<cnumb;i++) if (cl[i].cn == cn)
  {  
    cl[i].types[tn].defined = TRUE;
    strcpy(cl[i].types[tn].tname,name);
  }
}

/****
  CALLBACK ROUTINES
****/

void object_cb(FL_OBJECT *obj, long arg)
/* handles a class change event */
{
  if (fl_get_browser(obj))
    cur_class = cl[fl_get_browser(obj)-1].cn;
  else
    cur_class = -1;
}

/****
  INITIALIZING THE CLASSES
****/

int cur_class = -1;   /* The current class used */

void init_classes()
/* Initializes all the classes and types. Must be adapted
   to add new classes and types.
*/
{
  fl_add_browser_line(objectbrowser,"box");
  add_class_def(FL_BOX,"box",fl_create_box);
    add_type_def(FL_BOX,FL_NO_BOX,"NO_BOX");
    add_type_def(FL_BOX,FL_UP_BOX,"UP_BOX");
    add_type_def(FL_BOX,FL_DOWN_BOX,"DOWN_BOX");
    add_type_def(FL_BOX,FL_FLAT_BOX,"FLAT_BOX");
    add_type_def(FL_BOX,FL_BORDER_BOX,"BORDER_BOX");
    add_type_def(FL_BOX,FL_SHADOW_BOX,"SHADOW_BOX");
    add_type_def(FL_BOX,FL_FRAME_BOX,"FRAME_BOX");
    add_type_def(FL_BOX,FL_ROUNDED_BOX,"ROUNDED_BOX");
    add_type_def(FL_BOX,FL_RFLAT_BOX,"RFLAT_BOX");
    add_type_def(FL_BOX,FL_RSHADOW_BOX,"RSHADOW_BOX");

  fl_add_browser_line(objectbrowser,"text");
  add_class_def(FL_TEXT,"text",fl_create_text);
    add_type_def(FL_TEXT,FL_NORMAL_TEXT,"NORMAL_TEXT");

  fl_add_browser_line(objectbrowser,"bitmap");
  add_class_def(FL_BITMAP,"bitmap",fl_create_bitmap);
    add_type_def(FL_BITMAP,FL_NORMAL_BITMAP,"NORMAL_BITMAP");

  fl_add_browser_line(objectbrowser,"chart");
  add_class_def(FL_CHART,"chart",fl_create_chart);
    add_type_def(FL_CHART,FL_BAR_CHART,"BAR_CHART");
    add_type_def(FL_CHART,FL_HORBAR_CHART,"HORBAR_CHART");
    add_type_def(FL_CHART,FL_LINE_CHART,"LINE_CHART");
    add_type_def(FL_CHART,FL_FILLED_CHART,"FILLED_CHART");
    add_type_def(FL_CHART,FL_SPIKE_CHART,"SPIKE_CHART");
    add_type_def(FL_CHART,FL_PIE_CHART,"PIE_CHART");
    add_type_def(FL_CHART,FL_SPECIALPIE_CHART,"SPECIALPIE_CHART");

  fl_add_browser_line(objectbrowser,"@_clock");
  add_class_def(FL_CLOCK,"clock",fl_create_clock);
    add_type_def(FL_CLOCK,FL_SQUARE_CLOCK,"SQUARE_CLOCK");
    add_type_def(FL_CLOCK,FL_ROUND_CLOCK,"ROUND_CLOCK");

  fl_add_browser_line(objectbrowser,"button");
  add_class_def(FL_BUTTON,"button",fl_create_button);
    add_type_def(FL_BUTTON,FL_NORMAL_BUTTON,"NORMAL_BUTTON");
    add_type_def(FL_BUTTON,FL_PUSH_BUTTON,"PUSH_BUTTON");
    add_type_def(FL_BUTTON,FL_RADIO_BUTTON,"RADIO_BUTTON");
    add_type_def(FL_BUTTON,FL_TOUCH_BUTTON,"TOUCH_BUTTON");
    add_type_def(FL_BUTTON,FL_HIDDEN_BUTTON,"HIDDEN_BUTTON");
    add_type_def(FL_BUTTON,FL_INOUT_BUTTON,"INOUT_BUTTON");
    add_type_def(FL_BUTTON,FL_RETURN_BUTTON,"RETURN_BUTTON");
    add_type_def(FL_BUTTON,FL_HIDDEN_RET_BUTTON,"HIDDEN_RET_BUTTON");

  fl_add_browser_line(objectbrowser,"lightbutton");
  add_class_def(FL_LIGHTBUTTON,"lightbutton",fl_create_lightbutton);
    add_type_def(FL_LIGHTBUTTON,FL_NORMAL_BUTTON,"NORMAL_BUTTON");
    add_type_def(FL_LIGHTBUTTON,FL_PUSH_BUTTON,"PUSH_BUTTON");
    add_type_def(FL_LIGHTBUTTON,FL_RADIO_BUTTON,"RADIO_BUTTON");
    add_type_def(FL_LIGHTBUTTON,FL_TOUCH_BUTTON,"TOUCH_BUTTON");
    add_type_def(FL_LIGHTBUTTON,FL_HIDDEN_BUTTON,"HIDDEN_BUTTON");
    add_type_def(FL_LIGHTBUTTON,FL_INOUT_BUTTON,"INOUT_BUTTON");
    add_type_def(FL_LIGHTBUTTON,FL_RETURN_BUTTON,"RETURN_BUTTON");
    add_type_def(FL_LIGHTBUTTON,FL_HIDDEN_RET_BUTTON,"HIDDEN_RET_BUTTON");

  fl_add_browser_line(objectbrowser,"@_roundbutton");
  add_class_def(FL_ROUNDBUTTON,"roundbutton",fl_create_roundbutton);
    add_type_def(FL_ROUNDBUTTON,FL_NORMAL_BUTTON,"NORMAL_BUTTON");
    add_type_def(FL_ROUNDBUTTON,FL_PUSH_BUTTON,"PUSH_BUTTON");
    add_type_def(FL_ROUNDBUTTON,FL_RADIO_BUTTON,"RADIO_BUTTON");
    add_type_def(FL_ROUNDBUTTON,FL_TOUCH_BUTTON,"TOUCH_BUTTON");
    add_type_def(FL_ROUNDBUTTON,FL_HIDDEN_BUTTON,"HIDDEN_BUTTON");
    add_type_def(FL_ROUNDBUTTON,FL_INOUT_BUTTON,"INOUT_BUTTON");
    add_type_def(FL_ROUNDBUTTON,FL_RETURN_BUTTON,"RETURN_BUTTON");
    add_type_def(FL_ROUNDBUTTON,FL_HIDDEN_RET_BUTTON,"HIDDEN_RET_BUTTON");

  fl_add_browser_line(objectbrowser,"slider");
  add_class_def(FL_SLIDER,"slider",fl_create_slider);
    add_type_def(FL_SLIDER,FL_VERT_SLIDER,"VERT_SLIDER");
    add_type_def(FL_SLIDER,FL_HOR_SLIDER,"HOR_SLIDER");
    add_type_def(FL_SLIDER,FL_VERT_FILL_SLIDER,"VERT_FILL_SLIDER");
    add_type_def(FL_SLIDER,FL_HOR_FILL_SLIDER,"HOR_FILL_SLIDER");
    add_type_def(FL_SLIDER,FL_VERT_NICE_SLIDER,"VERT_NICE_SLIDER");
    add_type_def(FL_SLIDER,FL_HOR_NICE_SLIDER,"HOR_NICE_SLIDER");

  fl_add_browser_line(objectbrowser,"valslider");
  add_class_def(FL_VALSLIDER,"valslider",fl_create_valslider);
    add_type_def(FL_VALSLIDER,FL_VERT_SLIDER,"VERT_SLIDER");
    add_type_def(FL_VALSLIDER,FL_HOR_SLIDER,"HOR_SLIDER");
    add_type_def(FL_VALSLIDER,FL_VERT_FILL_SLIDER,"VERT_FILL_SLIDER");
    add_type_def(FL_VALSLIDER,FL_HOR_FILL_SLIDER,"HOR_FILL_SLIDER");
    add_type_def(FL_VALSLIDER,FL_VERT_NICE_SLIDER,"VERT_NICE_SLIDER");
    add_type_def(FL_VALSLIDER,FL_HOR_NICE_SLIDER,"HOR_NICE_SLIDER");

  fl_add_browser_line(objectbrowser,"dial");
  add_class_def(FL_DIAL,"dial",fl_create_dial);
    add_type_def(FL_DIAL,FL_NORMAL_DIAL,"NORMAL_DIAL");
    add_type_def(FL_DIAL,FL_LINE_DIAL,"LINE_DIAL");

  fl_add_browser_line(objectbrowser,"positioner");
  add_class_def(FL_POSITIONER,"positioner",fl_create_positioner);
    add_type_def(FL_POSITIONER,FL_NORMAL_POSITIONER,"NORMAL_POSITIONER");

  fl_add_browser_line(objectbrowser,"@_counter");
  add_class_def(FL_COUNTER,"counter",fl_create_counter);
    add_type_def(FL_COUNTER,FL_NORMAL_COUNTER,"NORMAL_COUNTER");
    add_type_def(FL_COUNTER,FL_SIMPLE_COUNTER,"SIMPLE_COUNTER");

  fl_add_browser_line(objectbrowser,"@_input");
  add_class_def(FL_INPUT,"input",fl_create_input);
    add_type_def(FL_INPUT,FL_NORMAL_INPUT,"NORMAL_INPUT");
    add_type_def(FL_INPUT,FL_FLOAT_INPUT,"FLOAT_INPUT");
    add_type_def(FL_INPUT,FL_INT_INPUT,"INT_INPUT");
    add_type_def(FL_INPUT,FL_HIDDEN_INPUT,"HIDDEN_INPUT");
    add_type_def(FL_INPUT,FL_MULTILINE_INPUT,"MULTILINE_INPUT");
    add_type_def(FL_INPUT,FL_SECRET_INPUT,"SECRET_INPUT");

  fl_add_browser_line(objectbrowser,"menu");
  add_class_def(FL_MENU,"menu",fl_create_menu);
    add_type_def(FL_MENU,FL_PUSH_MENU,"PUSH_MENU");
    add_type_def(FL_MENU,FL_TOUCH_MENU,"TOUCH_MENU");

  fl_add_browser_line(objectbrowser,"choice");
  add_class_def(FL_CHOICE,"choice",fl_create_choice);
    add_type_def(FL_CHOICE,FL_NORMAL_CHOICE,"NORMAL_CHOICE");

  fl_add_browser_line(objectbrowser,"@_browser");
  add_class_def(FL_BROWSER,"browser",fl_create_browser);
    add_type_def(FL_BROWSER,FL_NORMAL_BROWSER,"NORMAL_BROWSER");
    add_type_def(FL_BROWSER,FL_SELECT_BROWSER,"SELECT_BROWSER");
    add_type_def(FL_BROWSER,FL_HOLD_BROWSER,"HOLD_BROWSER");
    add_type_def(FL_BROWSER,FL_MULTI_BROWSER,"MULTI_BROWSER");

  fl_add_browser_line(objectbrowser,"timer");
  add_class_def(FL_TIMER,"timer",fl_create_timer);
    add_type_def(FL_TIMER,FL_NORMAL_TIMER,"NORMAL_TIMER");
    add_type_def(FL_TIMER,FL_VALUE_TIMER,"VALUE_TIMER");
    add_type_def(FL_TIMER,FL_HIDDEN_TIMER,"HIDDEN_TIMER");

/* ADD NEW CLASSES HERE (and in the routine add_an_object below). */
} 

/****
  GETTING INFORMATION ABOUT CLASSES AND TYPES
****/

char *find_class_name(int cln)
/* Returns a pointer to the name of the class */
{
  int i;
  for (i=0; i<cnumb; i++) if (cl[i].cn ==cln) return cl[i].cname;
  return NULL;
}

FL_OBJECT *find_class_default(int cln)
/* Returns a pointer to the default object of the class */
{
  int i;
  for (i=0; i<cnumb; i++) if (cl[i].cn ==cln) return cl[i].defobj;
  return NULL;
}

int find_class_maxtype(int cln)
/* Returns the number of types in the class */
{
  int i, j, n = 0;
  for (i=0; i<cnumb; i++) if (cl[i].cn ==cln)
      for (j=0; j<MAXTYPE; j++) if (cl[i].types[j].defined) n++;
  return n;
}

char *find_type_name(int cln,int tyn)
/* Returns a pointer to the name of the type in the class */
{
  int i;
  for (i=0; i<cnumb; i++) if (cl[i].cn ==cln) return cl[i].types[tyn].tname;
  return NULL;
}

/****
  ADDING OBJECTS
****/

#define bm_width 16
#define bm_height 16

static char bm_bits[] = {
   0xd3, 0x01, 0x85, 0x00, 0x93, 0x00, 0x95, 0x00, 0x93, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x88, 0x64, 0xd8, 0xa4, 0xa8, 0x6a, 0x88, 0x2e, 0x88, 0x2a,
   0x00, 0x00, 0xff, 0x03, 0x00, 0x00, 0xc0, 0xff};

FL_OBJECT *add_an_object(int objclass, int type,
			 float x, float y, float w, float h)
/* adds an object to the current form  type = -1 means default type */
{
  FL_OBJECT *obj;
  if (cur_form == NULL) return NULL;
  fl_addto_form(cur_form);
  switch(objclass)
  {
    case FL_BEGIN_GROUP:
	obj = fl_bgn_group(); break;
    case FL_END_GROUP: 
	obj = fl_end_group(); break;
    case FL_BOX:
        if (type == -1) type = FL_UP_BOX;
	obj = fl_add_box(type,x,y,w,h,""); break;
    case FL_TEXT:
        if (type == -1) type = FL_NORMAL_TEXT;
	obj = fl_add_text(type,x,y,w,h,"Text"); break;
    case FL_BITMAP:
        if (type == -1) type = FL_NORMAL_BITMAP;
	obj = fl_add_bitmap(type,x,y,w,h,"");
	fl_set_bitmap(obj,bm_width,bm_height,bm_bits);
	break;
    case FL_CHART:
        if (type == -1) type = FL_BAR_CHART;
	obj = fl_add_chart(type,x,y,w,h,"");
	fl_add_chart_value(obj,4.0,"item 1",2);
	fl_add_chart_value(obj,8.0,"item 2",3);
	fl_add_chart_value(obj,2.0,"item 3",4);
	fl_add_chart_value(obj,5.0,"item 4",5);
	break;
    case FL_CLOCK:
        if (type == -1) type = FL_SQUARE_CLOCK;
	obj = fl_add_clock(type,x,y,w,h,""); break;
    case FL_BUTTON:
        if (type == -1) type = FL_NORMAL_BUTTON;
	obj = fl_add_button(type,x,y,w,h,"Button"); break;
    case FL_LIGHTBUTTON:
        if (type == -1) type = FL_NORMAL_BUTTON;
	obj = fl_add_lightbutton(type,x,y,w,h,"Button"); break;
    case FL_ROUNDBUTTON:
        if (type == -1) type = FL_NORMAL_BUTTON;
	obj = fl_add_roundbutton(type,x,y,w,h,"Button"); break;
    case FL_SLIDER:
        if (type == -1) type = FL_VERT_SLIDER;
	obj = fl_add_slider(type,x,y,w,h,""); break;
    case FL_VALSLIDER:
        if (type == -1) type = FL_VERT_SLIDER;
	obj = fl_add_valslider(type,x,y,w,h,""); break;
    case FL_DIAL:
        if (type == -1) type = FL_NORMAL_DIAL;
	obj = fl_add_dial(type,x,y,w,h,""); break;
    case FL_POSITIONER:
        if (type == -1) type = FL_NORMAL_POSITIONER;
	obj = fl_add_positioner(type,x,y,w,h,""); break;
    case FL_COUNTER:
        if (type == -1) type = FL_NORMAL_COUNTER;
	obj = fl_add_counter(type,x,y,w,h,""); break;
    case FL_INPUT:
        if (type == -1) type = FL_NORMAL_INPUT;
	obj = fl_add_input(type,x,y,w,h,"Input"); break;
    case FL_MENU:
        if (type == -1) type = FL_PUSH_MENU;
	obj = fl_add_menu(type,x,y,w,h,"Menu");
	fl_addto_menu(obj,"item 1");
	fl_addto_menu(obj,"item 2");
	fl_addto_menu(obj,"item 3");
	fl_addto_menu(obj,"item 4");
	break;
    case FL_CHOICE:
        if (type == -1) type = FL_NORMAL_CHOICE;
	obj = fl_add_choice(type,x,y,w,h,"Choice");
	fl_addto_choice(obj,"item 1");
	fl_addto_choice(obj,"item 2");
	fl_addto_choice(obj,"item 3");
	fl_addto_choice(obj,"item 4");
	break;
    case FL_BROWSER:
        if (type == -1) type = FL_NORMAL_BROWSER;
	obj = fl_add_browser(type,x,y,w,h,"");
	fl_add_browser_line(obj,"Browser line 1");
	fl_add_browser_line(obj,"Browser line 2");
	fl_add_browser_line(obj,"Browser line 3");
	fl_add_browser_line(obj,"Browser line 4");
	break;
    case FL_TIMER:
        if (type == -1) type = FL_NORMAL_TIMER;
	obj = fl_add_timer(type,x,y,w,h,"Timer"); break;
    default:
	fl_end_form();
	fl_show_message("Error","","Trying to add unknow object type.");
	return NULL;
  }
  fl_end_form();
  obj->active = TRUE;
  return obj;
}
